//############################################################################
//
// LaserBoy !!!
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#ifndef __EZ_UAIL_FRAME_DEFINITIONS__
#define __EZ_UAIL_FRAME_DEFINITIONS__

//############################################################################
#include "ez_uail_segment.hpp"
#include "ez_real_segment.hpp"

//############################################################################
class ez_uail_frame : public ez_uail_header, public ez_uail_segment
{
public:
    //------------------------------------------------------------------------
    ez_uail_frame()
                 : ez_uail_header     (     ),
                   egg (0    ),
                   spider   (0    ),
                   is_selected        (false),
                   is_unique          (true )                     {}
    //------------------------------------------------------------------------
    ez_uail_frame(const ez_uail_frame& frame)
                 : ez_uail_header     ((ez_uail_header )frame),
                   ez_uail_segment    ((ez_uail_segment)frame),
                   egg (0                     ),
                   spider   (0                     ),
                   is_selected        (false                 ),
                   is_unique          (frame.is_unique       )    {}
   //-------------------------------------------------------------------------
    ez_uail_frame(const int& palette_index)
                 : ez_uail_header     (             ),
                   ez_uail_segment    (palette_index),
                   egg (0            ),
                   spider   (0            ),
                   is_selected        (false        ),
                   is_unique          (true         )             {}
    //------------------------------------------------------------------------
    ez_uail_frame(const int& palette_index, int not_used)
                 : ez_uail_header     (                       ),
                   ez_uail_segment    (palette_index, not_used),
                   egg (0                      ),
                   spider   (0                      ),
                   is_selected        (false                  ),
                   is_unique          (true                   )   {}
    //------------------------------------------------------------------------
    ez_uail_frame(const ez_uail_segment& segment, const ez_uail_header& header)
                 : ez_uail_header     (header ),
                   ez_uail_segment    (segment),
                   egg (0      ),
                   spider   (0      ),
                   is_selected        (false  ),
                   is_unique          (true   )                   {}
    //------------------------------------------------------------------------
    ez_uail_frame(const char&     format  ,
                  const string&   name    ,
                  const string&   owner   ,
                  const u_short&  qauntity,
                  const u_short&  identity,
                  const u_short&  total   ,
                  const char&     scanner
                 )
                 : ez_uail_header     (format  ,
                                       name    ,
                                       owner   ,
                                       quantity,
                                       identity,
                                       total   ,
                                       scanner ,
                                       0
                                      ),
                   egg (0    ),
                   spider   (0    ),
                   is_selected        (false),
                   is_unique          (true )                   {}
    //------------------------------------------------------------------------
    ez_uail_frame(ez_uail_point(*F)(int, int), int points_per_frame, int frame_index);
    //------------------------------------------------------------------------
virtual
   ~ez_uail_frame()                                             {}
    //------------------------------------------------------------------------
    void validate()
            {
                quantity = (u_short)number_of_points();                                 
                if(egg >= quantity)
                    egg = 0;
                if(spider >= quantity)
                    spider = 0;
            }
    //------------------------------------------------------------------------
    bool is_2D()
            {
                if(ez_uail_segment::is_2D())
                    format = EZ_UAIL_2D_VECTOR;
                else
                    format = EZ_UAIL_3D_VECTOR;
                return (bool)format; // 3D = 0, 2D = 1
            }
    //------------------------------------------------------------------------
    bool is_index_selected(int index) const
            {
                if(index >= 0 && index < number_of_points())
                {
                    if     (    egg == spider
                             && index == egg
                           )
                          return true;
                    else if(    egg <  spider
                             && index >= egg
                             && index <= spider
                           )
                          return true;
                    else if(     index <= egg
                              && index >= spider
                           )
                          return true;
                }
                return false;
            }
    //------------------------------------------------------------------------
    int  size_of_selection() const
            {
                if(egg == spider)
                    return 1;
                else if(egg <  spider)
                    return spider - egg;
                else 
                    return egg - spider;
            }
    //------------------------------------------------------------------------
    ez_uail_segment selected_segment() const
                        {
                            int             i;
                            ez_uail_segment segment;
                            segment.reserve(size());
                            for(i = 0; i < (int)size(); i++)
                                if(is_index_selected(i))
                                    segment += at(i);
                            return segment;
                        }
    //------------------------------------------------------------------------
    ez_uail_frame& operator =  (const ez_uail_frame& frame)
                    {
                        format        = frame.format       ;
                        quantity      = frame.quantity     ;
                        identity      = frame.identity     ;
                        total         = frame.total        ;
                        scanner       = frame.scanner      ;
                        future        = 0                  ;
                        error         = EZ_UAIL_OK         ;
                        palette_index = frame.palette_index;
                        clear();
                        reserve(frame.size());
                        insert(begin(), frame.begin(), frame.end());
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_uail_frame& operator =  (const ez_uail_header& header)
                    {
                        format    = header.format   ;
                        name      = header.name     ;
                        owner     = header.owner    ;
                        quantity  = header.quantity ;
                        identity  = header.identity ;
                        total     = header.total    ;
                        scanner   = header.scanner  ;
                        future    = 0               ;
                        error     = EZ_UAIL_OK      ;
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_uail_frame& operator =  (const ez_uail_segment& segment)
                    {
                        clear();
                        palette_index = segment.palette_index;
                        reserve(segment.size());
                        insert(begin(), segment.begin(), segment.end());
                        format    = is_2D();
                        quantity  = (u_short)number_of_points();
                        error     = EZ_UAIL_OK;
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_uail_frame& operator += (const ez_uail_frame& frame)
                    {
                        reserve(size() + frame.size());
                        if(palette_index != frame.palette_index)
                            promote_to_true_color();
                        insert(end(), frame.begin(), frame.end());
                        quantity = (u_short)number_of_points();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_uail_frame& operator += (const ez_uail_segment& segment)
                    {
                        reserve(size() + segment.size());
                        if(palette_index != segment.palette_index)
                            promote_to_true_color();
                        insert(end(), segment.begin(), segment.end());
                        quantity = (u_short)number_of_points();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_uail_frame& operator += (const ez_uail_point& point)
                    {
                        push_back(point);
                        quantity = (u_short)number_of_points();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_uail_frame  operator +  (const ez_uail_frame& frame) const
                    {
                        ez_uail_frame sum(*this);
                        sum += frame;
                        return sum;
                    }
    //------------------------------------------------------------------------
    ez_uail_frame& add_new_point    (int point_index);
    ez_uail_frame& break_segment    ();
    ez_uail_frame& connect_the_dots ();
    ez_uail_frame& to_dots          ();
    ez_uail_frame& selected_to_dots ();
    //------------------------------------------------------------------------
    void reduce_blank_vectors()
            {
                ez_uail_segment::reduce_blank_vectors();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    void remove_dots()
            {
                ez_uail_segment::remove_dots();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    void enhance_dots()
            {
                ez_uail_segment::enhance_dots();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    void remove_dwell_points()
            {
                ez_uail_segment::remove_dwell_points();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    void reduce_lit_vectors()
            {
                ez_uail_segment::reduce_lit_vectors();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    ez_uail_frame& reorder_segments()
                    {
                        ez_uail_segment::reorder_segments();
                        validate();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_uail_frame& fracture_segments()
                    {
                        ez_uail_segment::fracture_segments();
                        validate();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_uail_frame& bond_segments()
                    {
                        ez_uail_segment::bond_segments();
                        validate();
                        return *this;
                    }
    //------------------------------------------------------------------------
    ez_uail_frame& reverse()
                    {
                        ez_uail_segment::reverse();
                        validate();
                        return *this;
                    }
    //------------------------------------------------------------------------
    void add_dwell_points()
            {
                ez_uail_segment::add_dwell_points();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    void add_lit_span_points()
            {
                ez_uail_segment::add_lit_span_points();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    void add_blank_span_points()
            {
                ez_uail_segment::add_blank_span_points();
                validate();
                return;
            }
    //------------------------------------------------------------------------
    ez_uail_frame& remove_point(int point_index)
                    {
                        ez_uail_segment::remove_point(point_index);
                        validate();
                        return *this;
                    }
    //------------------------------------------------------------------------
    void  increment_egg()
            {
                egg++;
                if(egg >= quantity)
                    egg = 0;
            }
    //------------------------------------------------------------------------
    void  deccrement_egg()
            {
                egg--;
                if(egg < 0)
                    egg = quantity - 1;
            }
    //------------------------------------------------------------------------
    void  increment_spider()
            {
                spider++;
                if(spider >= quantity)
                    spider = 0;
            }
    //------------------------------------------------------------------------
    void  deccrement_spider()
            {
                spider--;
                if(spider < 0)
                    spider = quantity - 1;
            }
    //------------------------------------------------------------------------
    void save_as_ilda            (string file                               );
    void optimize                (                                          );
    int  from_ifstream_format_3  (ifstream& in, const ez_uail_header& header);
    int  from_ifstream_ilda      (ifstream& in, const ez_uail_header& header);
    int  from_ifstream_dxf       (ifstream& in, const ez_uail_header& header);
//    int  from_ifstream_shp       (ifstream& in, const ez_uail_header& header);
    int  move_selection          (ez_3D_float d                             );
    //------------------------------------------------------------------------
    int scale_selection                        (ez_3D_float m               );
    int scale_selection_around_point           (ez_3D_float d, ez_3D_float p);
    int scale_selection_around_fulcrum         (ez_3D_float d               );
    //------------------------------------------------------------------------
    int rotate_selection                       (ez_3D_float a               );
    int rotate_selection_around_point          (ez_3D_float a, ez_3D_float p);
    int rotate_selection_around_fulcrum        (ez_3D_float a               );
    //------------------------------------------------------------------------
    void next_segment_select      ();
    void next_segment_egg         ();
    void next_segment_spider      ();
    void previous_segment_select  ();
    void previous_segment_egg     ();
    void previous_segment_spider  ();
    //------------------------------------------------------------------------
    void recolor_select           (                                 )       ;
    void recolor_black_in_select  (                                 )       ;
    void blank_points             (                                 )       ;
    void unblank_points           (                                 )       ;
    void black_points             (                                 )       ;
    void unblack_points           (                                 )       ;
    void blank_equivalent_vectors (                                 )       ;    
    void minimize                 (                                 )       ;
    void to_ofstream_ilda         (ofstream& out                    )       ;
    void save_as_wave             (string file_name                 )       ;
    void to_ofstream_wave         (ofstream& out,
                                   ez_wave_header& header           )       ;
    void render_in_ezfb_bitmap    (string ui_message = ""           ) const ;
    void to_ofstream_code         (ofstream& out, const char* indent) const ;
    void save_as_code             (string file_name                 ) const ;
    void save_as_dxf              (string file_name                 ) const ;
    void save_as_bmp              (string file_name                 ) const ;
    void stats                    (bool show_cursors                ) const ;
    void display_stats            (                                 ) const ;
    //------------------------------------------------------------------------
    ez_uail_segment  transition_to_next_frame;

    int              egg   ,
                     spider;

    bool             is_selected,
                     is_unique  ;
};
//############################################################################
ez_uail_frame blank_frame();
ez_uail_frame NULL_frame ();

//############################################################################
#endif

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
